#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <pthread.h>
#include <signal.h>
#include "P2PTunnelAPIs.h"
#include "AVAPIs.h"
#include "IOTCAPIs.h"

#define MAX_SERVER_CONNECT_NUM		4
#define WEB_MAPPING_LOCAL_PORT		10000
#define SSH_MAPPING_LOCAL_PORT		20000
#define TELNET_MAPPING_LOCAL_PORT	30000
#define PASSWORD_MAPPING_LOCAL_PORT	40000
#define WEB_MAPPING_REMOTE_PORT		8080
#define SSH_MAPPING_REMOTE_PORT		22
#define TELNET_MAPPING_REMOTE_PORT	23
#define PASSWORD_MAPPING_REMOTE_PORT	9000

#define _ADD_AVAPI_FUNC_ 
#define _ADD_SOCKET_ 

char *gUID[MAX_SERVER_CONNECT_NUM];
int gProcessRun = 1;
int gWebIndex[MAX_SERVER_CONNECT_NUM];
int gTelnetIndex[MAX_SERVER_CONNECT_NUM];
int gSshIndex[MAX_SERVER_CONNECT_NUM];
int gPasswordIndex[MAX_SERVER_CONNECT_NUM];
int gRetryConnectFailedCnt[MAX_SERVER_CONNECT_NUM];

int gWebPort = WEB_MAPPING_LOCAL_PORT;

// default password if no "passwd.txt" exist
char gPassword[24]="P2P Platform";

typedef struct st_AuthData
{
	char szUsername[64];
	char szPassword[64];
} sAuthData;

typedef struct st_SessionHandler
{
	int nSID;
	int bException;
} sSessionHandler;

sSessionHandler gsSessionHandler[MAX_SERVER_CONNECT_NUM];

#ifdef _ADD_SOCKET_

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

void thread_connectsocket(void* arg)
{
	//客户端只需要一个套接字文件描述符，用于和服务器通信
	int clientSocket;
    //描述服务器的socket
	struct sockaddr_in serverAddr;
	char sendbuf[200]={"hello, this is a test !"};
	char recvbuf[200];
	int iDataNum;


	if((clientSocket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		perror("socket");
		return 1;
	}

	serverAddr.sin_family = AF_INET;
	serverAddr.sin_port = htons(WEB_MAPPING_LOCAL_PORT);
    //指定服务器端的ip，本地测试：127.0.0.1
    //inet_addr()函数，将点分十进制IP转换成网络字节序IP
	serverAddr.sin_addr.s_addr = inet_addr("127.0.0.1");
	if(connect(clientSocket, (struct sockaddr *)&serverAddr, sizeof(serverAddr)) < 0)
	{
		perror("connect");
		return 1;
	}

	printf("connect with destination host...\n");
	while(1)
	{
		send(clientSocket, sendbuf, strlen(sendbuf), 0);
		printf("send ok,sleep 3s\n");
		sleep(3);
	}
	close(clientSocket);
}
#endif

#ifdef _ADD_AVAPI_FUNC_

int InitAVAPIModule(int maxchannelnum)
{
	printf("do avInitialize(%d)\n",maxchannelnum);
	return avInitialize(maxchannelnum);
}
void deinitAVAPIModule()
{
	printf("do avDeInitialize()\n");
	avDeInitialize();
}

void thread_clientStart(void* arg)
{
	if(!arg){
		return ;
	}
	
	int sid = *(int*)arg;
	free(arg);
	if(sid < 0){
		return ;
	}
	unsigned int servType;
	int nResend;
	int avIndex = avClientStart2(sid, "admin","888888", 20, &servType, 1, &nResend);
	printf("avClientStart2 return %d\n",avIndex);
	if(avIndex < 0){
		printf("%s exit",__func__);
		return ;
	}
	unsigned int cmdType = 0x2883;
	int data = 100;
	while(1){
		int ret = avSendIOCtrl(avIndex, cmdType, (char*)&data, sizeof(int));
		if(ret < 0){
			break;
		}
		printf("avSendIOCtrl data succussfully,cmdType = 0x%x\n",cmdType);
		sleep(3);
	}
	
	printf("avServStop(%d)...\n",avIndex);
	avServStop(avIndex);	
	avIndex = -1;
}


#endif


int TunnelAgentStart(sAuthData *pAuthData)
{
	int ret;

	ret = P2PTunnelAgentInitialize(MAX_SERVER_CONNECT_NUM);
	if(ret < 0)
	{
		printf("P2PTunnelAgentInitialize error[%d]!\n", ret);
		return -1;
	}
	
#ifdef _ADD_AVAPI_FUNC_
	if(ret = InitAVAPIModule(32) < 0){
		printf("InitAVAPIModule error:%d\n",ret);
		return -1;
	}
#endif
	int i;
	for(i=0;i<MAX_SERVER_CONNECT_NUM;i++)
	{
		if(gUID[i] == NULL) continue;

		int nErrFromDevice = 0;
		/* If you don't want to use authentication mechanism, you can give NULL argument
		gsSessionHandler[i].nSID = P2PTunnelAgent_Connect(gUID[i], NULL, 0, &nErrFromDevice);
		*/
		gsSessionHandler[i].nSID = P2PTunnelAgent_Connect(gUID[i], (void *)pAuthData, sizeof(sAuthData), &nErrFromDevice);

		if(gsSessionHandler[i].nSID < 0)
		{
			printf("P2PTunnelAgent_Connect failed[%d], device respond reject reason[%d]\n", gsSessionHandler[i].nSID, nErrFromDevice);
			return -1;
		}
		else{
            printf("P2PTunnelAgent_Connect OK SID[%d]\n", gsSessionHandler[i].nSID);
#ifdef _ADD_AVAPI_FUNC_ // add AVAPI
			int *SID = (int *)malloc(sizeof(int));
			if(SID != NULL)
			{
				*SID = gsSessionHandler[i].nSID;
				pthread_t threadID;
				int ret = -1;
				if(ret = pthread_create(&threadID, NULL, &thread_clientStart, (void *)SID)){
					printf("clientstart Thread create fail, ret=[%d]\n", ret);
					return -1;
				}
				pthread_detach(threadID);
			}
#endif
		}

		if(P2PTunnel_SetBufSize(gsSessionHandler[i].nSID, 5120000) < 0)
			printf("P2PTunnel_SetBufSize error SID[%d]\n", gsSessionHandler[i].nSID);

		gWebIndex[gsSessionHandler[i].nSID] = P2PTunnelAgent_PortMapping(gsSessionHandler[i].nSID, WEB_MAPPING_LOCAL_PORT, WEB_MAPPING_REMOTE_PORT);
		if(gWebIndex[gsSessionHandler[i].nSID] < 0)
		{
			printf("P2PTunnelAgent_PortMapping WEB error[%d]!\n", gWebIndex[gsSessionHandler[i].nSID]);
		}
#ifdef _ADD_SOCKET_
		pthread_t threadID;
		int ret = -1;
		if(ret = pthread_create(&threadID, NULL, &thread_connectsocket, NULL)){
			printf("clientstart Thread create fail, ret=[%d]\n", ret);
			return -1;
		}
		pthread_detach(threadID);
#endif

	}

	return 0;
}

void TunnelStatusCB(int nErrorCode, int nSID, void *pArg)
{
    /* Don't do any Tunnel API call, because it maybe cause process dead lock. */

	if(nErrorCode == TUNNEL_ER_DISCONNECTED)
	{
		if(pArg != NULL)
			printf("MyArg = %s\n", (char *)pArg);

		int i;
		for(i=0;i<MAX_SERVER_CONNECT_NUM;i++)
		{
			if(gsSessionHandler[i].nSID == nSID)
			{
				gsSessionHandler[i].bException = 1;
				break;
			}
		}
	}
}

void TunnelStatusHandler(void)
{
	int i;
    for(i=0;i<MAX_SERVER_CONNECT_NUM;i++)
    {
		if(gsSessionHandler[i].bException)
		{
            int nSID = gsSessionHandler[i].nSID;

            P2PTunnelAgent_Disconnect(nSID);

            P2PTunnelAgent_StopPortMapping(gWebIndex[nSID]);
            P2PTunnelAgent_StopPortMapping(gTelnetIndex[nSID]);
            P2PTunnelAgent_StopPortMapping(gSshIndex[nSID]);
            P2PTunnelAgent_StopPortMapping(gPasswordIndex[nSID]);

            gsSessionHandler[i].nSID = -1;
            gsSessionHandler[i].bException = 0;
		}
	}
}

void InitGlobalArgument()
{
	int i;
	for(i=0;i<MAX_SERVER_CONNECT_NUM;i++)
	{
		gUID[i] = NULL;
		gsSessionHandler[i].nSID = -1;
		gsSessionHandler[i].bException = 0;
		gWebIndex[i] = -1;
		gSshIndex[i] = -1;
		gTelnetIndex[i] = -1;
		gPasswordIndex[i] = -1;
		gRetryConnectFailedCnt[i] = 0;
	}
}

int main(int argc, char *argv[])
{
	FILE *pfd = NULL;
	if(argc < 2)
	{
		printf("No UID input!!!\n");
		return 0;
	}

	InitGlobalArgument();

	int i, j = 0;
	for(i=1;i<argc;i++)
	{
		gUID[j] = (char *)malloc(21);
		if(gUID[j] != NULL)
		{
			strcpy(gUID[j], argv[i]);
			printf("SERVER UID[%s]\n", gUID[j]);
			j++;
		}
	}

	char *s = "My arg Pass to call back function";
	P2PTunnelAgent_GetStatus(TunnelStatusCB, (void *)s);

	sAuthData authData;
	strcpy(authData.szUsername, "Tutk.com");

	if( (pfd=fopen("passwd.txt","r")) !=NULL )
	{
		int len=0;
		memset(gPassword, 0, sizeof(gPassword));
		if ( fgets(gPassword, sizeof(gPassword), pfd) != NULL )
		{
		}

			len = strlen(gPassword);
		if(len>0 && gPassword[len-1]=='\n')
			gPassword[len-1] = '\0';
		fclose(pfd);
	}
	strcpy(authData.szPassword, gPassword);

	if(TunnelAgentStart(&authData) < 0)
	{
		printf("TunnelAgentStart failed\n");
		return -1;
	}

	while(gProcessRun)
	{
		int i;
		for(i=0;i<MAX_SERVER_CONNECT_NUM;i++)
		{
			if(gsSessionHandler[i].nSID >= 0)
			{
				int access_time = P2PTunnel_LastIOTime(gsSessionHandler[i].nSID);
				if(access_time >= 0)
					printf("SID %d:%u\n", gsSessionHandler[i].nSID, access_time);
			}
		}
		
		sleep(1);
		
	}
#ifdef _ADD_AVAPI_FUNC_
		deinitAVAPIModule();
#endif

	return 0;
}

